@@ -6,7 +6,6 @@
 #include "services/abstract/rootitem.h"
 
 #include "core/message.h"
-#include "core/messagefilter.h"
 #include "definitions/typedefs.h"
 
 #include <QJsonDocument>
@@ -19,6 +18,7 @@ class FeedsModel;
 class RecycleBin;
 class ImportantNode;
 class UnreadNode;
+class SearchsNode;
 class LabelsNode;
 class Label;
 class MessagesModel;
@@ -42,7 +42,11 @@ class ServiceRoot : public RootItem {
       Synchronised = 8
     };
 
-    enum class BagOfMessages { Read, Unread, Starred };
+    enum class BagOfMessages {
+      Read,
+      Unread,
+      Starred
+    };
 
   public:
     explicit ServiceRoot(RootItem* parent = nullptr);
@@ -52,6 +56,7 @@ class ServiceRoot : public RootItem {
     RecycleBin* recycleBin() const;
     ImportantNode* importantNode() const;
     LabelsNode* labelsNode() const;
+    SearchsNode* probesNode() const;
     UnreadNode* unreadNode() const;
 
     virtual void updateCounts(bool including_total_count);
@@ -67,6 +72,7 @@ class ServiceRoot : public RootItem {
     virtual QVariantHash customDatabaseData() const;
     virtual void setCustomDatabaseData(const QVariantHash& data);
     virtual bool wantsBaggedIdsOfExistingMessages() const;
+    virtual bool displaysEnclosures() const;
     virtual void aboutToBeginFeedFetching(const QList<Feed*>& feeds,
                                           const QHash<QString, QHash<ServiceRoot::BagOfMessages, QStringList>>&
                                             stated_messages,
@@ -199,7 +205,7 @@ class ServiceRoot : public RootItem {
     void completelyRemoveAllData();
 
     // Returns counts of updated messages <unread, all>.
-    QPair<int, int> updateMessages(QList<Message>& messages, Feed* feed, bool force_update, QMutex* db_mutex);
+    UpdatedArticles updateMessages(QList<Message>& messages, Feed* feed, bool force_update, QMutex* db_mutex);
 
     QIcon feedIconForMessage(const QString& feed_custom_id) const;
 
@@ -222,9 +228,20 @@ class ServiceRoot : public RootItem {
     QStringList textualFeedIds(const QList<Feed*>& feeds) const;
     QStringList customIDsOfMessages(const QList<ImportanceChange>& changes);
     QStringList customIDsOfMessages(const QList<Message>& messages);
-    QStringList customIDSOfMessagesForItem(RootItem* item);
 
-    void performInitialAssembly(const Assignment& categories, const Assignment& feeds, const QList<Label*>& labels);
+    // Returns list of article IDs depending on what target operation is.
+    // NOTE: So if we want to mark some articles as read,
+    // then we only return UNREAD IDs here to really return
+    // only IDs when the change makes sense.
+    // NOTE: Importance is not dealt here because it was not needed
+    // yet.
+    QStringList customIDSOfMessagesForItem(RootItem* item,
+                                           RootItem::ReadStatus target_read = RootItem::ReadStatus::Unknown);
+
+    void performInitialAssembly(const Assignment& categories,
+                                const Assignment& feeds,
+                                const QList<Label*>& labels,
+                                const QList<Search*>& probes);
 
   public slots:
     virtual void addNewFeed(RootItem* selected_item, const QString& url = QString());
@@ -255,11 +272,6 @@ class ServiceRoot : public RootItem {
     // from another machine and then performs sync-in on this machine.
     void removeLeftOverMessageFilterAssignments();
 
-    // Removes all labels/message assignments which are
-    // assigned to non-existing messages or which are
-    // assigned from non-existing labels.
-    void removeLeftOverMessageLabelAssignments();
-
     // Takes lists of feeds/categories and assembles them into the tree structure.
     void assembleCategories(const Assignment& categories);
     void assembleFeeds(const Assignment& feeds);
@@ -293,6 +305,7 @@ class ServiceRoot : public RootItem {
     RecycleBin* m_recycleBin;
     ImportantNode* m_importantNode;
     LabelsNode* m_labelsNode;
+    SearchsNode* m_probesNode;
     UnreadNode* m_unreadNode;
     int m_accountId;
     QList<QAction*> m_serviceMenu;
